-- ==============================================================
-- FILE : HE_RWCLK.VHD  -  DO NOT EDIT THIS FILE
-- ==============================================================
--
-- This file is part of the Hardware Interface Layer (H.I.L) of
-- your project when using your HERON-FPGA4 module.
--
-- This file must NOT be modified by users of the Module.
--
-- ==============================================================
--
--  Module      : HE_RWCLK
--  Date        : 28/02/2002
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : FIFO Clocks for the IN and OUT FIFOs
--
-- ==============================================================
--
--  Notes :
--  -1-  when FIFO read/write clock frequency > 60 MHz,
--       HFreq must be set to TRUE.
--  -2-  Using CLKDLL primitives (inspite of DCM for Virtex2)
--       for source code compatibilty with Spartan2.
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    28-02-02    First Written
--  1.1     R. Williams    08-05-02    Syntax change made to all
--                                     instanciated components,
--                                     (changed '0' to GND)
--                                     for compatibility with
--                                     ModelSim5.5b.
--  1.2     R. Williams    11-06-02    Added 1ns delay on paths
--                                     from inferred flip-flops
--                                     to instantiated flip-flops
--                                     to prevent setup violations
--                                     in functional simulation
--  2.0     R. Williams    11-11-02    New components added for
--                                     interfacing to all six HERON
--                                     input FIFOs and all six HERON
--                                     output FIFOs at the same time.
--                                     Added component HE_RD_6F in
--                                     place of component HE_RD_1F.
--                                     Added component HE_WR_6F in
--                                     place of component HE_WR_1F.
--  2.1     R. Williams    20-01-03    Changes made to the HSB
--                                     mastering functions, and 
--                                     additional registering added
--                                     to the AE flag to ensure
--                                     correct reading from HEPC8s.
--  2.2     R. Williams    12-04-05    Extended DLL reset to 3 clock
--                                     cycles
--  2.3     R. Williams    08-06-06    Merged ES and non-ES support.
--
-- ==============================================================


-- ==============================================================

--  Module      : HE_RCLK
--  Description : IN FIFO Clock

library IEEE;
  use IEEE.std_logic_1164.all;

-- synopsys translate_off
library UNISIM;
  use UNISIM.vcomponents.all;
-- synopsys translate_on

entity HE_RCLK is
  port (
    -- External PADS
    DCLK : in  std_logic;
    FCLK : out std_logic;
    -- Module I/Os
    RST  : in  std_logic;
    CLK  : in  std_logic;
    GCLK : out std_logic
  );
end HE_RCLK;


architecture RTL of HE_RCLK is

  component CLKDLLHF
    port (CLKIN  : in  std_logic;
          CLKFB  : in  std_logic;
          RST    : in  std_logic;
          CLK0   : out std_logic;
          CLK180 : out std_logic;
          CLKDV  : out std_logic;
          LOCKED : out std_logic);
  end component;

  component BUFG
    port(I : in  std_logic;
         O : out std_logic);
  end component;

  component FDP
    port(
      Q   : out std_logic;
      D   : in  std_logic;
      C   : in  std_logic;
      PRE : in  std_logic);
  end component;

  signal GND     : std_logic;
  signal R1      : std_logic;
  signal R2      : std_logic;
  signal R3      : std_logic;
  signal R4      : std_logic;
  signal R5      : std_logic;
  signal RST_DLL : std_logic;
  signal CLKIN   : std_logic;
  signal CLK0    : std_logic;
  signal GCLK0   : std_logic;

  attribute LOC : string;
  attribute LOC of iDLL : label is "DCM_X3Y0";

begin

  GND <= '0';

  FCLK <= CLK;    -- OBUF_F_24 (instanciation in the top level)

  CLKIN <= DCLK;  -- IBUFG (instanciation in the top level)

  process(CLK)
  begin
    if rising_edge(CLK) then
      R3 <= '0';
      if R2='1' and R1='0' then
        R3 <= '1';
      end if;
    end if;
  end process;
  process(CLK)
  begin
    if rising_edge(CLK) then
      R4 <= R3;
      R5 <= R4;
    end if;
  end process;
  process(CLK)
  begin
    if rising_edge(CLK) then
      RST_DLL <= '0';
      if R3='1' or R4='1' or R5='1' then
        RST_DLL <= '1';
      end if;
    end if;
  end process;

  iFDP0: FDP
    port map ( Q   => R1,
               D   => RST,
               C   => CLK,
               PRE => GND );

  iFDP1: FDP
    port map ( Q    => R2,
               D    => R1,
               C    => CLK,
               PRE  => GND );

  iDLL : CLKDLLHF
    port map ( CLKIN => CLKIN,
               CLKFB => GCLK0,
               RST   => RST_DLL,
               CLK0  => CLK0 );

  bGCK : BUFG port map (I=>CLK0, O=>GCLK0);
  GCLK <= GCLK0;

end RTL;

-- ==============================================================

--  Module      : HE_WCLK
--  Description : OUT FIFO Clock

library IEEE;
  use IEEE.std_logic_1164.all;

-- synopsys translate_off
library UNISIM;
  use UNISIM.vcomponents.all;
-- synopsys translate_on

entity HE_WCLK is
  port (
    -- External PADS
    DCLK : in  std_logic;
    FCLK : out std_logic;
    -- Module I/Os
    RST  : in  std_logic;
    CLK  : in  std_logic;
    GCLK : out std_logic
  );
end HE_WCLK;


architecture RTL of HE_WCLK is

  component CLKDLLHF
    port (CLKIN  : in  std_logic;
          CLKFB  : in  std_logic;
          RST    : in  std_logic;
          CLK0   : out std_logic;
          CLK180 : out std_logic;
          CLKDV  : out std_logic;
          LOCKED : out std_logic);
  end component;

  component BUFG
    port(I : in  std_logic;
         O : out std_logic);
  end component;

  component FDP
    port(
      Q   : out std_logic;
      D   : in  std_logic;
      C   : in  std_logic;
      PRE : in  std_logic);
  end component;

  signal GND     : std_logic;
  signal R1      : std_logic;
  signal R2      : std_logic;
  signal R3      : std_logic;
  signal R4      : std_logic;
  signal R5      : std_logic;
  signal RST_DLL : std_logic;
  signal CLKIN   : std_logic;
  signal CLK0    : std_logic;
  signal GCLK0   : std_logic;

  attribute LOC : string;
  attribute LOC of iDLL : label is "DCM_X4Y0";

begin

  GND <= '0';

  FCLK <= CLK;    -- OBUF_F_24 (instanciation in the top level)

  CLKIN <= DCLK;  -- IBUFG (instanciation in the top level)

  process(CLK)
  begin
    if rising_edge(CLK) then
      R3 <= '0';
      if R2='1' and R1='0' then
        R3 <= '1';
      end if;
    end if;
  end process;
  process(CLK)
  begin
    if rising_edge(CLK) then
      R4 <= R3;
      R5 <= R4;
    end if;
  end process;
  process(CLK)
  begin
    if rising_edge(CLK) then
      RST_DLL <= '0';
      if R3='1' or R4='1' or R5='1' then
        RST_DLL <= '1';
      end if;
    end if;
  end process;

  iFDP0: FDP
    port map ( Q   => R1,
               D   => RST,
               C   => CLK,
               PRE => GND );

  iFDP1: FDP
    port map ( Q    => R2,
               D    => R1,
               C    => CLK,
               PRE  => GND );

  iDLL : CLKDLLHF
    port map ( CLKIN => CLKIN,
               CLKFB => GCLK0,
               RST   => RST_DLL,
               CLK0  => CLK0 );

  bGCK : BUFG port map (I=>CLK0, O=>GCLK0);
  GCLK <= GCLK0;

end RTL;
